/*
DTMF receiver LCD monitor and Web uploader for WM series wind sensor

 This code use modified library of DTMF decoder, LiquidCrystal and Repeating Web client.

 Supported model: WM7-TM2

created 27 June 2016
 by Steve Nishiarai
*/

#define VERSION "171123"

// include the library code:
#include <stdlib.h>

/*--------------------------------------------------------------------------------
  Mode selections

  Use A2,A3,A4 pin for selecting mode. These pins are read as lower digit of ModeSel and each bits are reversed.
  Keep these pins open when not in the test mode.

  When ModeSel1, just show receiving DTMF tones to LCD.
  WM sensor side setting comand for this test in the 1_config.txt file is command M.

  When ModeSel2, send out DTMF test initiation tone once.
  If you desire to send test initiation tone again then try to push RESET button.

  When ModeSel3, show receiving DTMF tones to LCD and send back acknowledege tone.
  If the first letter in the DTMF data string is '0' or '2' then -NOM- tone will return.
  If the first letter in the DTMF data string is '1' then -OFF- tone will return.
  In other cases, no tone will return.
  WM sensor side setting comand for this test in the 1_config.txt file is command N.

  When ModeSel4 then web connection test will performed and show the received result on LCD.

|A4|A3|A2|
|--|--|Gd| ModeSel1 (show receiving DTMF tones)
|--|Gd|--| ModeSel2 (DTMF test initiation)
|--|Gd|Gd| ModeSel3 (show receiving DTMF tones, send back acknowledege)
|Gd|--|--| ModeSel4 (web connection test)
|Gd|--|Gd| ModeSel5 (no web connection = Monitor mode)
-- :open, Gd :Connect to GND
*/
//test mode register (0 = not test mode)
char ModeSel ;
//initialize TestStatus for making Web test only once
unsigned char TestStatus = 0 ;
//-------------------------------Control tone initialization>>
//assign Tone output to digital pin3
#define TONEOUT 3
//assign Relay control to digital pin2
#define PTT 2
//define TONENORMAL frequency in 667Hz. this is normal acknowledge tone
#define TONENORMAL 667
//define TONEOFF frequency in 1KHz. this is power off request tone
#define TONEOFF 1000
//define TONETEST frequency in 839Hz. this is test initiation tone
#define TONETEST 839
//define tone duration in milliseconds
#define TONEDUR 1000
//define test tone duration in milliseconds
#define TESTTONEDUR 2000
//tone control characters (fixed length of 5)
char ToneCont[] = "     " ;
//-----------------------------<<Control tone initialization
//----------------------------------------------------------------------------

/*------------------------------------------------------------------------------
  Watchdog timer.

  If the loop function does not run in 8 seconds then watchdog timer will reset the system.
  Watch dog timer function is disabled when the A5 pin is connected to the GND.
*/
#include <avr/wdt.h>
// Print loop time for debugging the watch dog timing
#define DEBUG_WDT
#ifdef DEBUG_WDT
unsigned long Looptime = 0 ;
#endif
//------------------------------------------------------------------------------

/*------------------------------------------------------------------------------
  Work hours.

  For the purpose of notify the receiver is running, system restarts in every work hours defined by WORKHOURS if A1 pin is not connected to GND.
  When "C" ID found in received DTMF codes, the start point of the work hours be adjusted.
  Time length of WORKHOURS is normally 24 hours.
*/
//Define work hours in milli seconds 24h = 24*3600*1000
#define WORKHOURS 86400000
//Register for memory the time when "C" ID found in received DTMF codes.
unsigned long WakeUpTime = 0;
//------------------------------------------------------------------------------

/*------------------------------------------------------------------------------
 * Receiving DTMF characters represents following wind information
 *
 * character count|content
 *        1       |   ID  
 *        2       |Average wind direction
 *        3       |Average wind speed tens place
 *        4       |Average wind speed ones place
 *        5       |Average wind speed after the decimal point
 *        6       |Maximum wind direction
 *        7       |Maximum wind speed tens place
 *        8       |Maximum wind speed ones place
 *        9       |Maximum wind speed after the decimal point
 *        10      |Check sum upper digit
 *        11      |Check sum lower digit
 *
 * If the ID is 0 then 1st wind information.
 * If the ID is from 1 to 9 and A then retry wind information.
 * Others are telemetry information.
*/
// include the library code:
#include <DTMF.h>
// Print DTMF code for debugging
//#define DEBUG_DTMF
//------------------------------------------------------------------------------

//--------------------------------------------------------------------DTMF initialization>>
//Assign pin A0 for DTMF decode sensor
int sensorPin = A0;
// NOTE that N MUST NOT exceed 160
// This is the number of samples which are taken in a call to
// .sample. The smaller the value of N the wider the bandwidth.
// For example, with N=128 at a sample rate of 8926Hz the tone
// detection bandwidth will be 8926/128 = 70Hz. If you make N
// smaller, the bandwidth increases which makes it harder to detect
// the tones because some of the energy of one tone can cross into
// an adjacent (in frequency) tone. But a larger value of N also means
// that it takes longer to collect the samples.
// A value of 64 works just fine, as does 128.
// NOTE that the value of N does NOT have to be a power of 2.
float n=128.0;
// sampling rate in Hz
float sampling_rate=8926.0;

// Instantiate the dtmf library with the number of samples to be taken
// and the sampling rate.
DTMF dtmf = DTMF(n,sampling_rate);

//int nochar_count = 0;
//last time you have received DTMF tone in milliseconds
unsigned long lastReceivedTime = 0;
//maximum expected space time in milliseconds (nominal space time is 40mS)
//#define DTMFSPACEMAX 100
#define DTMFSPACEMAX 500

float d_mags[8];

char pointer = 0 ;
//set boot up ID (11 letters are used for normal mode, 15 letters are used for test mode)
char DTMFhex[] = "C1" VERSION "0000000" ;

char SumErrCount = 0 ;
//--------------------------------------------------------------------<<DTMF initialization

/*--------------------------------------------------------------------
  LiquidCrystal Library
 
 Demonstrates the use a 16x2 LCD display.  The LiquidCrystal
 library works with all LCD displays that are compatible with the 
 Hitachi HD44780 driver. There are many of them out there, and you
 can usually tell them by the 16-pin interface.
 
 The circuit:
 * LCD RS pin to digital pin 9
 * LCD Enable pin to digital pin 8
 * LCD D4 pin to digital pin 7
 * LCD D5 pin to digital pin 6
 * LCD D6 pin to digital pin 5
 * LCD D7 pin to digital pin 4
 * LCD R/W pin to ground
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)

 Library originally added 18 Apr 2008
 by David A. Mellis
 library modified 5 Jul 2009
 by Limor Fried (http://www.ladyada.net)
 example added 9 Jul 2009
 by Tom Igoe
 modified 22 Nov 2010
 by Tom Igoe
 
 This example code is in the public domain.

 http://www.arduino.cc/en/Tutorial/LiquidCrystal
 --------------
 modified 18 Mar 2015
 by Steve Nishiarai
*/
// include the library code:
#include <LiquidCrystal.h>
// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(9, 8, 7, 6, 5, 4);
//define display buffer and pointer for WM test mode display
char DispBuff[16] ;
unsigned char DspPointer ;
//----------------------------------------------------------------

/*----------------------------------------------------------------
  Repeating Web client
 
 This sketch connects to a web server and makes a request
 using a Wiznet Ethernet shield. You can use the Arduino Ethernet shield, or
 the Adafruit Ethernet shield, either one will work, as long as it's got
 a Wiznet Ethernet module on board.
 
 This example uses DHCP, to get IP address, and DNS address.
 This example uses DNS, by assigning the Ethernet client with a MAC address, IP address, and DNS address.
 
 Circuit:
 * Ethernet shield attached to pins 10, 11, 12, 13
 
 created 19 Apr 2012
 by Tom Igoe
 
 http://arduino.cc/en/Tutorial/WebClientRepeating
 This code is in the public domain.
 
 --------------
 modified 28 Oct 2012
 by Steve Nishiarai
*/
// include the library code:
#include <SPI.h>
#include <Ethernet.h>

// assign a MAC address for the ethernet controller.
// fill in your address here:
//byte mac[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
byte mac[] = { 0x84, 0x83, 0xFE, 0xFF, 0x00, 0x01 }; //MAC address adjusted for #1

/*
//------------------->>IP address information when not using DHCP
// fill in the fixed IP address for this machine
IPAddress ip(0,0,0,0);

// fill in your Domain Name Server address here:
IPAddress myDns(0,0,0,0);
//------------------<<IP address information
*/

// initialize the library instance:
EthernetClient client;

// fill in your server domain name here:
//char server[] = "yourdomain.jp";
char server[] = "parawind.net";

//fill in your directory where your target CGI located under your domain:
//char dir[] = "location/" ;
char dir[] = "local/test/" ;

//define Wind information expiration time in milliseconds (11 minutes)
#define INFOEXPTIME 660000
// last time the wind information was shown on LCD, in milliseconds
unsigned long lastInfoTime = 0;
// last time you attempted connect to the server, in milliseconds
unsigned long lastAttemptTime = 0;

// state of the connection last time through the main loop
boolean lastConnected = false;

// delay between updates, in milliseconds
const unsigned long postingInterval = 10000;
// state of the connection success return from the httpRequest
boolean ConnectionSuccess = false;
// Print HTTP data for debugging
//#define DEBUG_HTTP
//----------------------------------------------------------------

void setup()
{
  // reset the WDT timer
  wdt_reset();
  // Disable watchdog timer
  WDTCSR |= (1<<WDCE) | (0<<WDE);
	//Initialize mode pins
	pinMode (A1, INPUT);
	digitalWrite(A1, HIGH);  // set pullup on pin A1
	pinMode (A2, INPUT);
	digitalWrite(A2, HIGH);  // set pullup on pin A2
	pinMode (A3, INPUT);
	digitalWrite(A3, HIGH);  // set pullup on pin A3
  pinMode (A4, INPUT);
  digitalWrite(A4, HIGH);  // set pullup on pin A4
  pinMode (A5, INPUT);
  digitalWrite(A5, HIGH);  // set pullup on pin A5
  // set tone out and PTT control pins
  pinMode( TONEOUT, OUTPUT) ;
  pinMode( PTT, OUTPUT) ;
  // start serial port:
  Serial.begin(9600);

	//read test mode pins
	ModeSel = 0 ;
	if ( digitalRead(A2) == 0 ) { ModeSel |= 0x01 ;}
	if ( digitalRead(A3) == 0 ) { ModeSel |= 0x02 ;}
	if ( digitalRead(A4) == 0 ) { ModeSel |= 0x04 ;}

  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);
  // Print a message to the LCD.
  lcd.print(F("DTMF"));
  lcd.print(VERSION);
	lcd.setCursor(0, 1);
  lcd.print(F("MAC"));
	for (char i=0; i<6; i++)
	{
		char c[6] ;
		sprintf ( c, "%02X", mac[i] ) ;
		lcd.print(c);
	}
	//put carsor to mode display  place
	lcd.setCursor(10, 0);

  //if test mode
	if ( ModeSel != 0 )
	{
	  //show mode selection
	  lcd.print(F(" Mode"));
	  char c = bin2ascii( ModeSel ) ;
	  lcd.print(c);
	}
  //if not test mode or test mode 4
	if ( ModeSel == 0 || ModeSel == 4 )
	{
	  // give the ethernet module time to boot up:
	  delay(1000);
//------------------------------------------------------------------>>get IP address and start Ethernet when using DHCP
	  lcd.print(F(" DHCP"));
	  if (Ethernet.begin(mac) == 0)
		{
	    Serial.println(F("Failed to configure Ethernet using DHCP"));
	    // no point in carrying on, so do nothing forevermore:
	    while(true);
	  }
//------------------------------------------------------------------<<get IP address and start Ethernet when using DHCP

//------------------------------------------------------------------>>Start Ethernet with fixed IP address when not using DHCP
//	  lcd.print(" FIXIP");
//	  // give the ethernet module time to boot up:
//	  delay(1000);
//	  Ethernet.begin(mac, ip, myDns) ; 
//------------------------------------------------------------------<<Start Ethernet with fixed IP address when not using DHCP

	  //clear the 2nd line
	  lcd.setCursor(0, 1);
	  lcd.print(F("                "));
		//show directory under the domain
		lcd.setCursor(0, 1);
		lcd.print(dir);

	  // give the Ethernet shield few seconds to initialize:
	  delay(2000);

	  // print the Ethernet board/shield's IP address:
	  Serial.print(F("My IP address: "));
	  Serial.println(Ethernet.localIP());
	  lcd.setCursor(0, 0);
	  lcd.print(F("My IP address"));
	  //clear the 2nd line
	  lcd.setCursor(0, 1);
	  lcd.print(F("                "));
	  lcd.setCursor(0, 1);
	  lcd.print(Ethernet.localIP());
	  delay(1000);

	  //Upload the startup indication repetitively until its success
	  int a = 1 ;
	  while (!ConnectionSuccess)
		{
	    lcd.setCursor(0, 0);
	    lcd.print(F("Connect attempt"));
	    lcd.print(a);
	    httpRequest();
	    //wait for posting interval
	    while (millis() - lastAttemptTime <= postingInterval) {}
	    a++;
		}
		lcd.setCursor(0, 0);
		lcd.print(F("                "));
		if ( digitalRead(A5) == 1 )
		{
			watchdogSetup();
		}
  }
}

void watchdogSetup(void)
{
  cli(); // disable all interrupts
  // reset the WDT timer
  wdt_reset();
/*
 WDTCSR configuration:
 WDIE = 1: Interrupt Enable
 WDE = 1 :Reset Enable
 WDP3 = 1 :For 8000ms Time-out
 WDP2 = 0 :For 8000ms Time-out
 WDP1 = 0 :For 8000ms Time-out
 WDP0 = 1 :For 8000ms Time-out
*/
  // Enter Watchdog Configuration mode:
  WDTCSR |= (1<<WDCE) | (1<<WDE);
  // Set Watchdog settings:
  WDTCSR = (1<<WDIE) | (1<<WDE) | (1<<WDP3) | (0<<WDP2) | (0<<WDP1) | (1<<WDP0);
  sei();
}

void software_reset()
{
  wdt_disable();
  wdt_enable(WDTO_15MS);
  while (1) {}
}

void loop()
{
#ifdef DEBUG_WDT
	Looptime = millis() - Looptime ;
	if ( Looptime > 1000 )
	{
		Serial.print(F("Debug "));
		Serial.print(millis()) ;
		Serial.print(F(" "));
		Serial.print(Looptime) ;
		Serial.println();
	}
	Looptime = millis() ;
#endif
	if ( digitalRead(A5) == 1 )
	{
		//Make sure the proccess is in loop
		wdt_reset();
	}
	//If time comes to Work hours with A1 pin in periodic mode then reset the system.
	unsigned long RestartTime = WORKHOURS + WakeUpTime ;
	if ( millis() > RestartTime && digitalRead(A1) == 1 )
	{
		software_reset();
	}
	//--- if DTMF test initiation mode then
	while ( ModeSel == 2 )
	{
		//send out test initiation tone
		if ( digitalRead(A3) == 0 )
		{
      digitalWrite(PTT, HIGH) ;     //PTT on
      tone(TONEOUT, TONETEST, TESTTONEDUR) ;
      delay(TESTTONEDUR);           // wait for tone out
      digitalWrite(PTT, LOW) ;      //PTT off
	    // test initiation tone be sent only once, so do nothing forevermore:
	    while(true);
		}
	}
	//--- if WM connection test mode then 
	if ( ModeSel == 4 )
	{
		if ( TestStatus == 0 )
		{
	    Serial.println(F("Test mode"));
		  lcd.print(F("Test mode"));
			lcd.setCursor(0, 1);
		  // close any connection before send a new request.
		  client.stop();
		  // if there's a successful connection:
		  if (client.connect(server, 80))
			{
#ifdef DEBUG_HTTP
		    Serial.println(server);
		    Serial.println(F("connecting....."));
#endif
		    // send the HTTP directory:
		    client.print(F("GET /"));
		    client.print(dir);
		    client.print(F("wmtest.html"));
		    client.println(F(" HTTP/1.1"));
		    client.print(F("Host: "));
		    client.println(server);
		    client.println(F("User-Agent: arduino-ethernet"));
		    client.println(F("Connection: close"));
		    client.println();
#ifdef DEBUG_HTTP
		    Serial.println(F("HTTP test request sent."));
#endif
		    ConnectionSuccess = true ;
				TestStatus = 1 ;
		  } 
		  else
			{
		    // if you couldn't make a connection:
#ifdef DEBUG_HTTP
		    Serial.println(F("connection failed"));
#endif
		    ConnectionSuccess = false;
		  }
			lcd.setCursor(0, 1);
		}
	  if (client.available())
		{
	    char c = client.read();
#ifdef DEBUG_HTTP
	    Serial.write(c);
#endif
			//put letter to the LCD display buffer
			if ( c == '\n' )
			{
				lcd.setCursor(0, 0);
				for ( unsigned int i = 0; i < 16; i++ )
				{
					lcd.print(DispBuff[i]) ;
				}
			  //clear the 2nd line
			  lcd.setCursor(0, 1);
			  lcd.print(F("                "));
				lcd.setCursor(0, 1);
				DspPointer = 0 ;
			}
			else if ( DspPointer < 16 )
			{
				DispBuff[DspPointer] = c ;
				lcd.print(c) ;
				DspPointer++ ;
			}
		}
	}
	//--- end of WM connection test mode
	else
	{
	  //if not test mode
		if ( ModeSel == 0 )
		{
		  // if there's incoming data from the net connection.
		  // shift it in the tone control characters
		  if (client.available())
			{
		    char c = client.read();
#ifdef DEBUG_HTTP
		    Serial.write(c);
#endif
		    ToneCont[4] = ToneCont[3] ;
		    ToneCont[3] = ToneCont[2] ;
		    ToneCont[2] = ToneCont[1] ;
		    ToneCont[1] = ToneCont[0] ;
		    ToneCont[0] = c;
		    //if the power off command of "-OFF-" was detected in the HTTP response then send OFF tone signal
		    if (ToneCont[4] == '-' && ToneCont[3] == 'O' && ToneCont[2] == 'F' && ToneCont[1] == 'F' && ToneCont[0] == '-') 
				{
		      digitalWrite(PTT, HIGH) ;     //PTT on
		      delay(1000);                  // wait for a second
		      tone(TONEOUT, TONEOFF, TONEDUR) ;
		      delay(TONEDUR);                  // wait for tone out
		      digitalWrite(PTT, LOW) ;      //PTT off
		    }
		    //if the indication of normal response "-NOM-" was detected in the HTTP response then send normal tone signal
		    if (ToneCont[4] == '-' && ToneCont[3] == 'N' && ToneCont[2] == 'O' && ToneCont[1] == 'M' && ToneCont[0] == '-')
				{
		      digitalWrite(PTT, HIGH) ;     //PTT on
		      delay(1000);                  // wait for a second
		      tone(TONEOUT, TONENORMAL, TONEDUR) ;
		      delay(TONEDUR);                  // wait for tone out
		      digitalWrite(PTT, LOW) ;      //PTT off
		    }
		  }
		  // if there's no net connection, but there was one last time
		  // through the loop, then stop the client:
		  if (!client.connected() && lastConnected)
			{
#ifdef DEBUG_HTTP
		    Serial.println();
		    Serial.println(F("disconnecting."));
#endif
		    client.stop();
		  }
		}
	  // receive DTMF dial tones
	  char thischar;
	  
	  // This reads N samples from sensorpin (must be an analog input)
	  // and stores them in an array within the library. Use while(1)
	  // to determine the actual sampling frequency as described in the
	  // comment at the top of this file
	  /* while(1) */dtmf.sample(sensorPin);
	  
	  // The first argument is the address of a user-supplied array
	  // of 8 floats in which the function will return the magnitudes
	  // of the eight tones.
	  // The second argument is the value read by the ADC when there
	  // is no signal present. A voltage divider with precisely equal
	  // resistors will presumably give a value of 511 or 512.
	  // My divider gives a value of 506.
	  // If you aren't sure what to use, set this to 512
	//  dtmf.detect(d_mags,506);
	  dtmf.detect(d_mags,512);
	
	  // detect the button
	  // If it is recognized, returns one of 0123456789ABCD*#
	  // If unrecognized, returns binary zero
	
	  // Pass it the magnitude array used when calling .sample
	  // and specify a magnitude which is used as the threshold
	  // for determining whether a tone is present or not
	  //
	  // If N=64 magnitude needs to be around 1200
	  // If N=128 the magnitude can be set to 1800
	  // but you will need to play with it to get the right value
	  thischar = dtmf.button(d_mags,1800.);
	
	  //if not test mode or test mode 5
		if ( ModeSel == 0 || ModeSel == 5 )
		{
			//show vital sign with *
	    lcd.setCursor(9, 0);
			if (( (millis() / 1000 ) & 0x1 ) == 0 )
			{
		    lcd.print(F("*"));
			}
			else
			{
				//clear *
			  lcd.print(F(" "));
			}
		  //when a code was detected
		  if(thischar)
			{
		    //display received codes
#ifdef DEBUG_DTMF
		    Serial.print(thischar); //
#endif
	      // set the cursor to column pointer, line 0
	      lcd.setCursor(pointer, 0);
		    lcd.print(thischar);
		    //nochar_count = 0;
		    lastReceivedTime = millis() ;
		    DTMFhex[pointer] = thischar ;
		    if ( ++pointer == 11 )
				{
		      SendDTMF() ;
		      pointer = 0 ;
					//if test mode5
					if ( ModeSel == 5 )
					{
						//Send Ack tone at this timing
			      delay(2000);                  // wait for 2 second
			      digitalWrite(PTT, HIGH) ;     //PTT on
			      delay(1000);                  // wait for a second
			      tone(TONEOUT, TONENORMAL, TONEDUR) ;
			      delay(TONEDUR);                  // wait for tone out
			      digitalWrite(PTT, LOW) ;      //PTT off
					}
		    }
	// Print the magnitudes for debugging
//#define DEBUG_PRINT
#ifdef DEBUG_PRINT
		    for(int i = 0;i < 8;i++)
				{
		      Serial.print(F("  "));
		      Serial.print(d_mags[i]);
		    }
		    Serial.println("");
#endif
			}
		  //when any code was not detected
		  else
			{
				//if the wind information is expired then
				if ( millis() > lastInfoTime + INFOEXPTIME )
				{
					//set cursor to 2nd line
		      lcd.setCursor(0, 1);
					//clear 2nd line
			    lcd.print(F("                "));
				}
		    //if the space time is longer than maximum expected time then
		    if ( pointer > 0 && millis() > lastReceivedTime + DTMFSPACEMAX )
				{
		      // print a newline 
#ifdef DEBUG_DTMF
		      Serial.println(""); //-debug
#endif
		      // set the cursor to column 0, line 0
		      lcd.setCursor(0, 0);
					//clear the DTMF codes in the 1st line, but leave checksum counter.
			    lcd.print(F("              "));
		      // set the cursor to column 0, line 0
		      lcd.setCursor(0, 0);
		      //purge DTMF buffer
		      pointer = 0 ;
		    }
			}
		}
	  //if it is test mode 1 or 3
		else
		//Check whether some code comming
		{
		  //Put sapce for indicate null to the code buffer
			char CmdN = ' ' ;
		  if(thischar)
			{
				//just show DTMF characters
		    lcd.print(thischar);
		    //nochar_count = 0;
		    lastReceivedTime = millis() ;
		    DTMFhex[pointer] = thischar ;
				//stop the pointer at the end for avoiding overrun
		    if ( ++pointer == 15 ) { pointer = 14 ;}
			}
		  //when any code was not detected and if the space time is longer than maximum expected time then
		  else if ( pointer > 0 && millis() > lastReceivedTime + DTMFSPACEMAX )
			{
	      // print a newline 
#ifdef DEBUG_HTTP
	      Serial.println("");
#endif
				//clear 1st line
		    lcd.print(F("                "));
				//copy previous data to the 1st line
	      lcd.setCursor(0, 0);
				//save first character
				CmdN = DTMFhex[0] ;
				for (int i=0; i<15; i++)
				{
			    lcd.print(DTMFhex[i]);
					//clear the previous buffer
					DTMFhex[i] = ' ' ;
				}
				//set cursor to 2nd line
	      lcd.setCursor(0, 1);
				//clear 2nd line
		    lcd.print(F("                "));
				//set cursor again
	      lcd.setCursor(0, 1);
	      pointer = 0 ;
			}
		  //if it is test mode 3 and 1st letter is in buffer then send back tone signal
			if ( ModeSel == 3 && CmdN != ' ' )
			{
				int toneBuf = TONENORMAL ;
				//if the command letter is 1 then send -OFF- tone
				if ( CmdN == '1' )
				{
		      toneBuf = TONEOFF ;
				}
	      digitalWrite(PTT, HIGH) ;     //PTT on
	      delay(2000);                  // waits for two second
	      tone(TONEOUT, toneBuf, TONEDUR) ;
	      delay(TONEDUR);               // wait for tone out
	      digitalWrite(PTT, LOW) ;      //PTT off
			}
		}
	  // don't let it wrap around
	  if ( millis() < lastReceivedTime ) lastReceivedTime = 0 ;
	  // store the state of the connection for next time through
	  lastConnected = client.connected();
	}
}

void SendDTMF()
{
  //convert ascii characters to hexadecimal code (use only lower digit)
  for (int i=0; i<11; i++)
	{
    if ( DTMFhex[i] == '#' ) { DTMFhex[i] = 'E' ;}
    if ( DTMFhex[i] == '*' ) { DTMFhex[i] = 'F' ;}
    DTMFhex[i] = ascii2bin(DTMFhex[i]) ;
  }
  //convert checksum code to 8 bit binary
  char chksum = DTMFhex[9]*16 | DTMFhex[10] ;
  //generate higher digit of converted checksum display character
  char ChkSumDsp1 = chksum / 16 ;
  ChkSumDsp1 = bin2ascii( ChkSumDsp1 ) ;
  //generate lower digit of converted checksum display character
  char ChkSumDsp2 = bin2ascii( chksum ) ;
  //check checksum
  char CalChksum = 0 ;
  for(int i=0; i<9; i++)
	{
    //generate calculated checksum
    CalChksum += DTMFhex[i] ;
  }
	chksum -= CalChksum ;
  //if check sum was not OK then 
  if ( chksum != 0 )
	{
    SumErrCount++ ;
  }
  //initialize display pointer for showing hexa codes
  lcd.setCursor(0, 0);
  //backup wind direction codes
  char AvrDir = DTMFhex[1] ;
  char MaxDir = DTMFhex[5] ;
  for(int i = 0;i < 9;i++)
	{
    DTMFhex[i] = bin2ascii( DTMFhex[i] ) ;
    //show converted hexa codes
    lcd.print(DTMFhex[i]);
  }
  lcd.print(F(" "));
  //show converted checksum
  lcd.print(ChkSumDsp1);
  lcd.print(ChkSumDsp2);
  //show calculated checksum
  ChkSumDsp1 = CalChksum / 16 ;
  ChkSumDsp1 = bin2ascii( ChkSumDsp1 ) ;
  ChkSumDsp2 = bin2ascii( CalChksum ) ;
  lcd.print(ChkSumDsp1);
  lcd.print(ChkSumDsp2);
  //show checksum counter
  char c = SumErrCount / 16 ;
  c = bin2ascii( c ) ;
  //show upper digit
  lcd.print( c );
  c = bin2ascii( SumErrCount ) ;
  //show lower digit
  lcd.print( c );
  //if check sum was OK then 
  if ( chksum == 0 )
	{
	  //if not test mode
		if ( ModeSel == 0 )
		{
	    //upload DTMF data
	    // if you're not connected, and ten seconds have passed since
	    // your last attempt of connection, then connect again and send data:
	    if(!client.connected() && (millis() - lastAttemptTime > postingInterval))
			{
	      httpRequest();
	    }
		}
    lcd.setCursor(0, 1);
    // since the ID is converted to ascii charatcter, check whether it is number column
    char c = DTMFhex[0] & 0xF0 ;
    //ID was wind data then
    if ( c == 0x30 || DTMFhex[0] == 'A' )
		{
      //show wind data to second row
      DispWinDir( AvrDir ) ;
      //suppress zero
      if ( DTMFhex[2] == '0' ) DTMFhex[2] = ' ' ;
      lcd.print(DTMFhex[2]);
      lcd.print(DTMFhex[3]);
      lcd.print(F("."));
      lcd.print(DTMFhex[4]);
      lcd.print(F("  "));
      DispWinDir( MaxDir ) ;
      //suppress zero
      if ( DTMFhex[6] == '0' ) DTMFhex[6] = ' ' ;
      lcd.print(DTMFhex[6]);
      lcd.print(DTMFhex[7]);
      lcd.print(F("."));
      lcd.print(DTMFhex[8]);
      //reset last information time
      lastInfoTime = millis() ;
    }
    //If ID was wakeup code then
    if ( DTMFhex[0] == 'C' )
		{
			//Set wakeup time
			WakeUpTime = millis() ;
			//Adjust wake up time 10 minute before
			if ( WakeUpTime > 600000 ) { WakeUpTime -= 600000 ;} else { WakeUpTime = 0 ;}
		}
  }
}

void DispWinDir( char chr )
{
  switch( chr )
	{
    case 0: lcd.print(F("  N")); break ;
    case 1: lcd.print(F("NNE")); break ;
    case 2: lcd.print(F(" NE")); break ;
    case 3: lcd.print(F("ENE")); break ;
    case 4: lcd.print(F("  E")); break ;
    case 5: lcd.print(F("ESE")); break ;
    case 6: lcd.print(F(" SE")); break ;
    case 7: lcd.print(F("SSE")); break ;
    case 8: lcd.print(F("  S")); break ;
    case 9: lcd.print(F("SSW")); break ;
    case 10: lcd.print(F(" SW")); break ;
    case 11: lcd.print(F("WSW")); break ;
    case 12: lcd.print(F("  W")); break ;
    case 13: lcd.print(F("WNW")); break ;
    case 14: lcd.print(F(" NW")); break ;
    case 15: lcd.print(F("NNW")); break ;
  }
}

char bin2ascii( char chr )
{
  chr &= 0x0f ;
  if ( chr < 10 )
	{
    chr |= 0x30 ;
  }
  else
	{
    chr += 0x37 ;
  }
  return chr ;
}

unsigned char ascii2bin(unsigned char input)
{
	//if the bit6 is 1 then 
	if ( input >= 0x40 )
	{
		//it must be 0x0A - 0x0F
		input += 9 ;
	}
	input &= 0x0F ;
	return input ;
}

// this method makes a HTTP connection to the server:
void httpRequest()
{
  // close any connection before send a new request.
  // This will free the socket on the WiFi shield
  client.stop();
  // if there's a successful connection:
  if (client.connect(server, 80))
	{
#ifdef DEBUG_HTTP
    Serial.println(server);
    Serial.println(F("connecting..."));
#endif
    // send the HTTP PUT request:
    client.print(F("GET /"));
    client.print(dir);
    client.print(F("wlog.cgi?DTMF="));
		for ( int i=0; i<9; i++ )
		{
	    client.print(DTMFhex[i]);
		}
    client.println(F(" HTTP/1.1"));
    client.print(F("Host: "));
    client.println(server);
    client.println(F("User-Agent: arduino-ethernet"));
    client.println(F("Connection: close"));
    client.println();

    ConnectionSuccess = true ;
  } 
  else
	{
    // if you couldn't make a connection:
#ifdef DEBUG_HTTP
    Serial.println(F("connection failed"));
#endif
    ConnectionSuccess = false;
  }
  // note the time that the attempt of connection was made:
  lastAttemptTime = millis();
}
